-module(dist_app).

%% see erl -man application
-behaviour(application).

-export([start/2,
	 start_phase/3,
	 stop/1]).


%% This is the entry module for your application. It contains the
%% start function and some other stuff. You identify this module
%% using the 'mod' attribute in the .app file.


%% The start function is called by the application controller.
%% It normally returns {ok,Pid}, i.e. the same as gen_server and
%% supervisor. Here, we simply call the start function in our supervisor.
%% One can also return {ok, Pid, State}, where State is reused in stop(State).
%%
%% Type can be 'normal' or {takeover,FromNode}. If the 'start_phases'
%% attribute is present in the .app file, Type can also be {failover,FromNode}.
%% This is an odd compatibility thing.
%%
start(Type, _Args) ->
    io:format("dist_app:start(~p, _)~n", [Type]),
    dist_super:start_link().


%% We can also implement the function start_phase(Phase, Type, Args),
%% but this function will only be called if the .app file contains a
%% 'start_phases' attribute. See erl -man application for more details.
%% We will actually implement one start phase here: takeover.
start_phase(takeover, {takeover, FromNode}, Args) ->
    io:format("dist_app:start_phase(takeover, {takeover,~p}, _)~n",
	      [FromNode]),
    dist_server:takeover(FromNode),
    ok;
start_phase(takeover, _, Args) ->
    io:format("dist_app:start_phase(takeover, _)~n", []),
    %% not a takeover case -- do nothing.
    ok;
start_phase(go, Type, Args) ->
    io:format("dist_app:start_phase(go, _)~n", []),
    dist_server:go(Type).

%% Another function that could be implemented here is prep_stop(State),
%% but it is optional. Omitting it will not cause your application to crash.
%% If prep_stop(S) is implemented, it is called before shutting down the
%% processes of your application. Typically, prep_stop/1 could make sure that
%% ports are closed before servers needed for processing incoming messages
%% are killed.

%% stop(State) is called when the application has been terminated, and
%% all processes are gone. The return value is ignored.
stop(S) ->
    ok.


